@@ -3,7 +3,7 @@
 
 void rgl_assert (const char* assertion, const char* file, int line)
 {
-    error("Assertion failure: %s\nFile: %s\nLine: %d\nPlease report to rgl maintainer.",
+    Rf_error("Assertion failure: %s\nFile: %s\nLine: %d\nPlease report to rgl maintainer.",
            assertion, file, line);
 }
 
